import unittest

from smarttest.decorators import test_type


@test_type('acceptance')
class SomeAcceptanceTestCase(unittest.TestCase):

    def test_some_acceptance_test(self):
        pass


@test_type('unit')
class SomeUnitTestCase(unittest.TestCase):

    def test_some_unit_test(self):
        pass


class UnspecifiedTypeTestCase(unittest.TestCase):

    def test_some_test(self):
        pass



# -*- coding: utf-8 -*-
import glob
import os
import sys

from paver.easy import *  # noqa
try:
    import paver.doctools  # noqa
except ImportError:
    pass
from paver.setuputils import setup
sys.path.append('.')

with open('README.rst') as file:
    long_description = file.read()

setup(
    name='django-kidoredis',
    version='1.0.1',
    description='Django redis backend',
    long_description=long_description,
    url="http://kidosoft.pl",
    author="Jakub STOLARSKI (Dryobates)",
    author_email="jakub.stolarski@kidosoft.pl",
    license="beerware",
    keywords="django redis",
    packages=['kidoredis'],
    install_requires=[
        'redis',
        'hash_ring',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)

options(
    sphinx=Bunch(
        builddir="_build",
        apidir=None,
    )
)


@task
@needs('generate_setup', 'minilib', 'setuptools.command.sdist')
def sdist():
    """Overrides sdist to make sure that our setup.py is generated."""
    pass


@task
def cleanup():
    """ Removes generated files. """
    sh('rm -rf dist', ignore_error=True)
    sh('rm -rf build', ignore_error=True)
    sh('rm -rf *.egg-info', ignore_error=True)
    sh('rm -rf htmlcov', ignore_error=True)
    sh('rm -rf docs/_build', ignore_error=True)
    sh('rm setup.py', ignore_error=True)
    sh('rm paver-minilib.zip', ignore_error=True)


@task
def kwalitee(options):
    """ Removes generated files. """
    sh('flake8 %s' % ' '.join(options.packages), ignore_error=True)
    sh('pep257 %s' % ' '.join(options.packages), ignore_error=True)


@task
@task
@cmdopts([
    ("packages_dir=", "p", "Run tests with all Django versions found in directory"),
])
def coverage(options):
    """ Generages coverage reports. """
    omit = [
        'pavement.py',
        'manage.py',
        'setup.py',
        'example/*',
    ]
    omit = ','.join([repr(filename) for filename in omit])
    if getattr(options, 'packages_dir', False):
        sh('coverage erase')
        for filename in glob.glob(os.path.join(options.packages_dir, 'Django*')):
            sh('pip uninstall -q -y Django', ignore_error=True)
            sh('pip install -q -U %s' % filename)
            sh('coverage run --append --source . --omit %s manage.py test -v 0 %s' % (omit, ' '.join(options.packages)))
    else:
        sh('coverage run --source . --omit %s manage.py test -v 0 %s' % (omit, ' '.join(options.packages)))
    sh('coverage html')
    sh('coverage report --fail-under=100')


@task
@cmdopts([
    ("path=", "p", "Docs path"),
])
@needs('cleanup')
def publish_docs(options):
    """ Uploads docs to server. """
    try:
        call_task('html')
    except:
        pass
    sh('''sed -i '' 's/href="\(http:\/\/sphinx-doc.org\)/rel="nofollow" href="\\1"/' docs/_build/html/*.html''')
    sh('rsync -av docs/_build/html/ %s/%s/' % (options.path, options.name))



"""
Django settings for example project.

For more information on this file, see
https://docs.djangoproject.com/en/1.7/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.7/ref/settings/
"""
# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.7/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '4-aol-=3jk=^d%ai$xf95gpx^)0@0tf&g5(0uxp*d#-%7**^u%'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'kidoredis',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'example.urls'

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_URL = '/static/'
LOGIN_URL = '/admin/'

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    },
    'redis_ring': {
        'BACKEND': 'kidoredis.backends.RedisRing',
        'DB': '0',
        'LOCATION': [
            'localhost:6379',
            'localhost:6380',
        ]
    },
    'redis_copy': {
        'BACKEND': 'kidoredis.backends.RedisCopy',
        'DB': '0',
        'LOCATION': [
            'localhost:6379',
            'localhost:6380',
        ]
    },
}



# -*- coding: utf-8 -*-
""" Compatybility module for django version < 1.3. """

import importlib

import django

if django.VERSION < (1, 3):

    from django.conf import settings
    from django.core.cache import InvalidCacheBackendError

    def get_cache(backend, **kwargs):
        """ Return redis client based on alias in CACHES.

        @param backend: alias in CACHES dictionary.
        @returns: redis client
        """
        return _create_cache(backend, **kwargs)

    def _create_cache(backend, **kwargs):
        try:
            # Try to get the CACHES entry for the given backend name first
            try:
                conf = settings.CACHES[backend]
            except KeyError:
                try:
                    backend, cls_name = backend.rsplit('.', 1)
                    mod = importlib.import_module(backend)
                    backend_cls = getattr(mod, cls_name)
                except (ImportError, AttributeError) as e:
                    msg = "Could not find backend '%s': %s" % (backend, e)
                    raise InvalidCacheBackendError(msg)
                location = kwargs.pop('LOCATION', '')
                params = kwargs
            else:
                params = conf.copy()
                params.update(kwargs)
                backend = params.pop('BACKEND')
                location = params.pop('LOCATION', '')
                backend, cls_name = backend.rsplit('.', 1)
                mod = importlib.import_module(backend)
                backend_cls = getattr(mod, cls_name)
        except (ImportError, AttributeError) as e:
            msg = "Could not find backend '%s': %s" % (backend, e)
            raise InvalidCacheBackendError(msg)
        return backend_cls(location, params)
else:
    import warnings
    warnings.warn('In Django>1.2 please use django.core.cache.get_cache')
    from django.core.cache import get_cache


__all__ = ('get_cache',)



# -*- coding: utf-8 -*-

""" Backends which simplify access to redis through django cache framework. """

__doc__ = """
Backends
========
"""


from django.core.cache.backends.base import BaseCache
import hash_ring
import redis


class ConstRedis(redis.StrictRedis):

    """ Redis client that adds consistent hashing to StrictRedis. """

    def __init__(self, *args, **kwargs):
        self.hash_key = kwargs.pop('hash_key', 1)
        super(ConstRedis, self).__init__(*args, **kwargs)

    def __hash__(self):
        return self.hash_key

    def __str__(self):
        return '%s' % self.hash_key


class AbstractRedis(BaseCache):

    def __init__(self, location, params):
        self.db = params.pop('DB')
        super(AbstractRedis, self).__init__(params)
        self.create_nodes(location)

    def create_nodes(self, location):
        """ Create connections to redis nodes.

        @param str location: host:port indicating redis node location.
        """
        nodes = []
        for i, node in enumerate(location):
            host, port = node.rsplit(':', 1)
            redis_pool = redis.ConnectionPool(host=host, port=port, db=self.db)
            nodes.append(ConstRedis(connection_pool=redis_pool, hash_key=i))
        self.nodes = nodes
        self.nodes_ring = hash_ring.HashRing(nodes)


WRITE_COMMANDS = [
    'append',
    'bitop',
    'blpop',
    'brpop',
    'brpoplpush',
    'decr',
    'decrby',
    'del',
    'expire',
    'expireat',
    'flushall',
    'flushdb',
    'getset',
    'hdel',
    'hincrby',
    'hincrbyfloat',
    'hmset',
    'hset',
    'hsetnx',
    'incr',
    'incrby',
    'incrbyfloat',
    'linsert',
    'lpop',
    'lpush',
    'lpushx',
    'lrem',
    'lset',
    'ltrim',
    'mset',
    'msetnx',
    'persist',
    'pexpire',
    'pexpireat',
    'pfadd',
    'psetex',
    'rename',
    'renamenx',
    'rpop',
    'rpoplpush',
    'rpush',
    'rpushx',
    'sadd',
    'sdiffstore',
    'set',
    'setbit',
    'setex',
    'setnx',
    'setrange',
    'sinterstore',
    'smove',
    'sort',
    'spop',
    'srem',
    'sunionstore',
    'zadd',
    'zincrby',
    'zinterstore',
    'zrem',
    'zremrangebylex',
    'zremrangebyrank',
    'zremrangebyscore',
    'zunionstore',
]


class RedisRing(AbstractRedis):

    """ RedisRing backend which writes only to one Redis server based on key.

    RedisRing is better suited for using Redis as memcached replacement.
    With one server failure data kept on that server have to be recreated.
    """

    def __getattribute__(self, name):
        """ Proxy for StrictRedis attributes. """
        if name in ['create_nodes', 'nodes', 'nodes_ring', 'db']:
            return object.__getattribute__(self, name)

        def wrapper(*args, **kwargs):
            try:
                key = args[0]
            except IndexError:
                key = ''
            node = self.nodes_ring.get_node(key)
            func = getattr(node, name)
            return func(*args, **kwargs)
        return wrapper


class RedisCopy(AbstractRedis):

    """ Redis backend which writes to all Redis servers.

    Fetching is done only from one server based on key just like in RedisRing.
    RedisCopy can be seen like master/master configuration
    where synchronization isn't done by Redis itself but by clients.
    It's well suited for storing data that should be available
    even if one server goes down.
    """

    def __getattribute__(self, name):
        """ Proxy for StrictRedis attributes. """
        if name in ['create_nodes', 'nodes', 'nodes_ring', 'db']:
            return object.__getattribute__(self, name)

        def wrapper(*args, **kwargs):
            try:
                key = args[0]
            except IndexError:
                key = ''
            if name not in WRITE_COMMANDS:
                node = self.nodes_ring.get_node(key)
                func = getattr(node, name)
                return func(*args, **kwargs)
            else:
                results = []
                for node in self.nodes:
                    func = getattr(node, name)
                    results.append(func(*args, **kwargs))
                return results
        return wrapper


__all__ = ('RedisCopy', 'RedisRing')



from django.test import TestCase
from kidotest.decorators import no_db_testcase

from mock import patch, call
import redis

from ..backends import RedisRing, RedisCopy


@no_db_testcase
class RedisRingGetTest(TestCase):

    """
    :py:meth:`kidoredis.backends.RedisRing.get`
    """

    def test_should_get_value(self):
        key = 'test'
        location = [
            'localhost:6379',
            'localhost:6380',
        ]
        params = {'DB': 0}
        with patch.object(redis.StrictRedis, 'execute_command') as execute_command:
            client = RedisRing(location, params)
            client.get(key)
            execute_command.assert_called_once_with('GET', key)

    def test_should_set_value(self):
        key = 'test'
        value = 123
        location = [
            'localhost:6379',
            'localhost:6380',
        ]
        params = {'DB': 0}
        with patch.object(redis.StrictRedis, 'execute_command') as execute_command:
            client = RedisRing(location, params)
            client.set(key, value)
            execute_command.assert_called_once_with('SET', key, value)

    def test_should_get_info(self):
        location = [
            'localhost:6379',
            'localhost:6380',
        ]
        params = {'DB': 0}
        with patch.object(redis.StrictRedis, 'execute_command') as execute_command:
            client = RedisRing(location, params)
            client.info()
            execute_command.assert_called_once_with('INFO')


@no_db_testcase
class RedisCopyGetTest(TestCase):

    """
    :py:meth:`kidoredis.backends.RedisCopy.get`
    """

    def test_should_get_value(self):
        key = 'test'
        location = [
            'localhost:6379',
            'localhost:6380',
        ]
        params = {'DB': 0}
        with patch.object(redis.StrictRedis, 'execute_command') as execute_command:
            client = RedisCopy(location, params)
            client.get(key)
            execute_command.assert_called_once_with('GET', key)

    def test_should_set_value(self):
        key = 'test'
        value = 123
        location = [
            'localhost:6379',
            'localhost:6380',
        ]
        params = {'DB': 0}
        with patch.object(redis.StrictRedis, 'execute_command') as execute_command:
            client = RedisCopy(location, params)
            client.set(key, value)
            calls = [call('SET', key, value), call('SET', key, value)]
            execute_command.assert_has_calls(calls)

    def test_should_get_info(self):
        location = [
            'localhost:6379',
            'localhost:6380',
        ]
        params = {'DB': 0}
        with patch.object(redis.StrictRedis, 'execute_command') as execute_command:
            client = RedisCopy(location, params)
            client.info()
            execute_command.assert_called_once_with('INFO')



import unittest

from smarttest.decorators import test_type

class SomeAcceptanceTestCase(unittest.TestCase):
    """ Test method. """

    def setUp(self):
        super(SomeAcceptanceTestCase, self).setUp()
        self.mocker = Mocker()

    def tearDown(self):
        super(SomeAcceptanceTestCase, self).tearDown()

    def test_should_name(self):
        """ Scenariusz: Opis scenariusza """
        # Zakładając,
        # Jeżeli,
        self.mocker.replay()
        try:
            result = method(params)
        finally:
            self.mocker.restore()
        # Wtedy
        self.mocker.verify()




from django.conf import settings
from django.test import TestCase

from kidotest.decorators import no_db_testcase

from ..backends import RedisRing


@no_db_testcase
class GetCacheTest(TestCase):

    """
    :py:meth:`kidoredis.comap.get_cache`
    """

    def test_should_return_cache_by_alias(self):
        from kidoredis.compat import get_cache

        client = get_cache('redis_ring')

        self.assertIsInstance(client, RedisRing)

    def test_should_return_cache_by_path(self):
        from kidoredis.compat import get_cache

        client = get_cache('kidoredis.backends.RedisRing', DB=0)

        self.assertIsInstance(client, RedisRing)

    def test_should_raise_exception_on_nonexisting_backend(self):
        try:
            from kidoredis.compat import get_cache, InvalidCacheBackendError
        except ImportError:
            from django.core.cache import get_cache, InvalidCacheBackendError

        self.assertRaises(InvalidCacheBackendError, get_cache, 'kidoredis.backends.WrongBackend', DB=0)

    def test_should_raise_exception_on_wrong_backend_path(self):
        try:
            from kidoredis.compat import get_cache, InvalidCacheBackendError
        except ImportError:
            from django.core.cache import get_cache, InvalidCacheBackendError

        self.assertRaises(InvalidCacheBackendError, get_cache, 'kidoredis.wrong_module.RedisRing', DB=0)

    def test_should_raise_exception_on_wrong_alias(self):
        try:
            from kidoredis.compat import get_cache, InvalidCacheBackendError
        except ImportError:
            from django.core.cache import get_cache, InvalidCacheBackendError

        settings.CACHES['wrong_backend'] = {
            'BACKEND': 'kidoredis.backends.WrongBackend',
            'DB': '0',
            'LOCATION': [
                'localhost:6379',
                'localhost:6380',
            ]
        }
        try:
            self.assertRaises(InvalidCacheBackendError, get_cache, 'wrong_backend')
        finally:
            del settings.CACHES['wrong_backend']



try:
    import paver.tasks
except ImportError:
    from os.path import exists
    if exists("paver-minilib.zip"):
        import sys
        sys.path.insert(0, "paver-minilib.zip")
    import paver.tasks

paver.tasks.main()
